# ファイル名：common.pm
# 機能：
# 共通処理を纏めたモジュール
# 
# 改訂履歴：
# 1.0.0 ：新規作成      2007/11/10

package common;

use strict;
use lib qw( ./pm );
use system_config;
use DBI;
use CGI;
use HTML::Template;
use utf8;

# 関数名:check_string
# 機能  :文字列のチェック
# 引数  :なし
# 戻り値:NULLならば、空文字を返す
sub check_string {
	my($tmp) = @_;
	return defined($tmp)? $tmp: "";
}

# 関数名:check_number
# 機能  :文字列のチェック
# 引数  :なし
# 戻り値:NULLならば、空文字を返す
sub check_number {
	my($tmp) = @_;
	if(defined($tmp)) {
		if( $tmp =~ /[^0-9]/) {
			return 'all';
		} else {
			return int($tmp);
		}
	} else {
		return 0;
	}
	return defined($tmp)? int($tmp): 0;
}

# 関数名:check_date
# 機能  :日付のチェック
# 引数  :なし
# 戻り値:入力された日付が正しければ入力値、空文字の場合は0を返す
#		 異常値の場合はステータスコード400を返し終了する
sub check_date {
	my ($date) = @_;
	
	unless ($date eq "" || $date =~ /^\d{8}$/) {
		my $query = new CGI;
		
		print $query->header(-type=>'text/plane', -status=>'400 Bad Request', -charset=>'utf-8');
		exit(255);
	}

	return ($date eq "" ? 0 : $date);
}

# 関数名: check_referer
# 機能  : リファラーのチェック
# 引数  : CGIオブジェクト
# 戻り値: なし
sub check_referer {
	my ($query) = @_;

	unless ($query->referer() eq $::referer || $query->referer() eq $::referer_full) {
		warn "referer error";
		print $query->header(-type => 'text/plane', -status => '400 Bad request', -charset => 'utf-8');
		exit(1);
	}
}

# 関数名:db_open
# 機能  :DB接続
# 引数  :なし
# 戻り値:$dbh:データベースハンドル
sub db_open {
	my $dbh = DBI->connect("DBI:mysql:$::dbname:$::dbport;", 
		$::dbuser, $::dbpass, { AutoCommit => 0, RaiseError => 0, PrintError => 0 })
		or db_connect_error();
	$dbh->do("set names utf8");	

	return($dbh);
}

# 関数名: db_error
# 機能  : エラーメッセージの出力
# 引数  : なし
# 戻り値: なし
sub db_connect_error {
	# 標準出力への表示
	warn "db connect error";

	# 利用画面へのエラー表示
	my $query = new CGI;
	
	print $query->header(-type => 'text/plane', -status => '500 Internal server error', -charset => 'utf-8');
	print "db connect error.";

	exit(1);
}

# 関数名: db_abote_and_rollback
# 機能  : エラーメッセージの出力
# 引数  : $dbh	 	: データベースハンドル
#		  $sql		: sql文
#		  $filename	: sql実行ファイル名
#		  $html_temp: HTMLテンプレートのファイル名
#		  $position	: メッセージ表示位置
#		  $message	: 表示メッセージ
#		  @bind		: プレースホルダに入る値
# 戻り値: なし
sub db_abote_and_rollback {
	my ($dbh, $sql, $program, $template_file, $position, $message, @bind) = @_;

	$dbh->rollback;
	$dbh->disconnect;

	my $loop = 1;
	my $count = 0;
	while($loop){
		$loop = $sql =~ s/\?/$bind[$count]/m;
		$count++;
	}
	
	$sql =~ s/\s+/ /gm;

	warn "File: $program\nSQL: $sql\n";

	# 利用画面へのエラー表示
	my $query = new CGI;
	if($template_file && $position && $message){
		my $template = HTML::Template->new(filename => $template_file);
		$template->param($position => $message);

		print $query->header(-type=>'text/html', -charset=>'utf-8');
		print $template->output;
	} else {
		print $query->header(-type=>'text/plane',  -status=>'500 Internal server error', -charset=>'utf-8');
		print "db execute error."
	}

	exit(1);
}

# 関数名: execute_sql
# 機能  : プリペアを利用したSQL実行
# 引数  : $dbi		: データベースハンドル
#		  $filename	: 呼び出し元のファイル名
#		  $template	: HTMLテンプレートのファイル名
#		  $position	: テンプレート内メッセージ表示位置
#		  $message	: 表示メッセージ
#		  $sql		: SQL文
#		  $data		: 検索ワード
# 戻り値: $sth		: ステートメントハンドル
sub execute_sql {
	my($dbh, $sql, $filename, $template, $position, $message, @data) = @_;

	my $sth = $dbh->prepare($sql)
				or &db_abote_and_rollback($dbh, $sql, $filename, $template, $position, $message, @data);
	
	$sth->execute(@data) or
		&db_abote_and_rollback($dbh, $sql, $filename, $template, $position, $message, @data);

	return($sth);
}

# 関数名: count_sql_result
# 機能  : 実行したSQL文の結果件数を返す
# 引数  : $dbh		: データベースハンドル
#		  $sql		: SQL文
# 		  $filename	: 呼び出し元のファイル名
#		  @data		: SQL利用変数格納関数
# 戻り値: $sth:ステートメントハンドル $cnt:検索件数
sub count_sql_result {
	my($dbh, $sql, $filename, @data) = @_;
	my($rc);
	my($cnt);

	my $sth = $dbh->prepare($sql) or
				&db_abote_and_rollback($dbh, $sql, $filename, 0, 0, 0, @data);
	$rc = $sth->execute(@data) or 
				&db_abote_and_rollback($dbh, $sql, $filename, 0, 0, 0, @data);
	
	# 取得データ数判定
	if($rc eq '0E0'){   # 取得データ数=0
		$cnt = 0;
	} else{
		$cnt = $sth->rows;
	}
	return($cnt, $sth);
}

# 関数名：keyword_quote
# 機能  ：文字列をMySQL類似検索用の条件文字列に変換する。
# 引数  ：検索条件となる文字列
# 戻り値：変換された文字列
sub keyword_quote {
	my ($str_sql)= @_;

	# 1文字ワイルドカードを意味する「_」を「\_」に変換
	$str_sql =~ s/_/\\_/g;

	# ワイルドカードを意味する「%」を「\%」に変換
	$str_sql =~ s/%/\\%/g;

	# quote実行(後に%を付加)
	$str_sql = '%'.$str_sql.'%';
	return($str_sql);
}

# 関数名:substitute
# 機能  :文字列の置換を行う
# 引数  :input 処理対象文字列　pattern 置換する文字列　replacement 置換後の文字列
# 戻り値:置換処理後の文字列
sub substitute {
	my $input  = shift(@_);
	my $pattern = shift(@_);
	my $replacement = shift(@_);
	#print "$input\n$pattern\n$replacement\n";
	$input =~ s/$pattern/$replacement/g;
	return ($input);
}

# 関数名:HTMLEscape
# 機能  :HTML出力用に文字の置換を行う
# & -> &amp; < -> &lt; > -> &gt; " -> &quot; &'-> &#39;
# 引数  :input 置換対象文字列
# 戻り値:置換後の文字列
sub HTMLEscape {
	my $input = shift(@_);
	if(!defined($input)) {
		return "";
	}
	
	$input = &substitute($input, "&(?!amp;)", "&amp;");
	$input = &substitute($input, "<", "&lt;");
	$input = &substitute($input, ">", "&gt;");
	$input = &substitute($input, "\"", "&quot;");
	$input = &substitute($input, "'", "&#39;");
	return ($input);
}
1;
